from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, HttpResponseRedirect, HttpResponseForbidden
from django.shortcuts import get_object_or_404
from django.template import RequestContext, loader
from django.utils.translation import ugettext as _
from django.views.generic.list_detail import object_list, object_detail

from lava_data_store.models import Project 
from lava_data_store.models import Release
from lava_data_store.forms import ReleaseForm 


def release_list(request, project_id):
    project = get_object_or_404(
        Project.objects.accessible_by_principal(request.user), 
        pk=project_id)
    return object_list(
        request,
        queryset=project.releases.all().order_by('expected_date'),
        template_name="lava_data_store/release_list.html",
        template_object_name="release",
        extra_context={'project': project})


def release_detail(request, project_id, release_id):
    project = get_object_or_404(
        Project.objects.accessible_by_principal(request.user), 
        pk=project_id)
    release = get_object_or_404(
        project.releases.all(),
        pk=release_id)
    template_name = "lava_data_store/release_detail.html"
    t = loader.get_template(template_name)
    c = RequestContext(request, {
        'project': project,
        'release': release,
        'belongs_to_user': project.is_owned_by(request.user),
    })
    return HttpResponse(t.render(c))


@login_required
def release_create(request, project_id):
    project = get_object_or_404(
        Project.objects.accessible_by_principal(request.user), 
        pk=project_id)
    if request.method == 'POST':
        form = ReleaseForm(request.POST, request.FILES)
        if form.is_valid():
            release = Release.objects.create(
                project=project,
                name=form.cleaned_data['name'],
                expected_date=form.cleaned_data['expected_date'])
            return HttpResponseRedirect(release.get_absolute_url())
    else:
        form = ReleaseForm()
    template_name = "lava_data_store/release_form.html"
    t = loader.get_template(template_name)
    c = RequestContext(request, {
        'form': form,
        'submit_text': _(u"Create release")
    })
    return HttpResponse(t.render(c))


@login_required
def release_update(request, project_id, release_id):
    project = get_object_or_404(
        Project.objects.accessible_by_principal(request.user), 
        pk=project_id)
    if not project.is_owned_by(request.user):
        return HttpResponseForbidden()
    release = get_object_or_404(
        project.releases.all(),
        pk=release_id)
    if request.method == 'POST':
        form = ReleaseForm(request.POST, request.FILES)
        if form.is_valid():
            release.name= form.cleaned_data['name']
            release.expected_date = form.cleaned_data['expected_date']
            release.save()
            return HttpResponseRedirect(release.get_absolute_url())
    else:
        form = ReleaseForm(initial=dict(
            name=release.name,
            expected_date=release.expected_date))
    template_name = "lava_data_store/release_form.html"
    t = loader.get_template(template_name)
    c = RequestContext(request, {
        'form': form,
        'submit_text': _(u"Update release")
    })
    return HttpResponse(t.render(c))
